searchData={"items":[{"type":"module","title":"AKSPrimalityTest","doc":"Implementation of the AKS (Agrawal-Kayal-Saxena) primality test algorithm.\n\nThe AKS primality test is a deterministic polynomial-time algorithm for determining\nwhether a given number is prime. It was the first polynomial-time primality test\nthat is both general and deterministic.","ref":"AKSPrimalityTest.html"},{"type":"module","title":"Algorithm Overview - AKSPrimalityTest","doc":"The AKS algorithm works in the following steps:\n\n1. **Perfect Power Test**: Check if n is a perfect power ($a^b$ for some $a > 1$, $b > 1$)\n2. **Order Finding**: Find the smallest r such that the order of n modulo r is greater than $\\log_2 n$\n3. **GCD Test**: Check that n is coprime with all numbers up to r\n4. **Polynomial Test**: Verify that $(X + a)^n \\equiv X^n + a (\\pmod X^r - 1, n)$ for certain values of a","ref":"AKSPrimalityTest.html#module-algorithm-overview"},{"type":"module","title":"Time Complexity - AKSPrimalityTest","doc":"The algorithm runs in $O((\\log n)^{12})$ time, making it polynomial but not practical\nfor large numbers compared to probabilistic tests like Miller-Rabin.","ref":"AKSPrimalityTest.html#module-time-complexity"},{"type":"module","title":"Examples - AKSPrimalityTest","doc":"iex> AKSPrimalityTest.of(2)\n    true\n\n    iex> AKSPrimalityTest.of(3)\n    true\n\n    iex> AKSPrimalityTest.of(4)\n    false\n\n    iex> AKSPrimalityTest.of(17)\n    true\n\n    iex> AKSPrimalityTest.of(25)\n    false\n\n    iex> AKSPrimalityTest.of(127)\n    true\n\n    iex> AKSPrimalityTest.of(128)\n    false\n\n    iex> AKSPrimalityTest.of(8191)\n    true","ref":"AKSPrimalityTest.html#module-examples"},{"type":"module","title":"Dependencies - AKSPrimalityTest","doc":"This implementation depends on the following modules:\n- `PerfectPower` - for perfect power detection\n- `LehmerGcd` - for GCD calculations\n- `PrimeFactorization` - for prime factorization","ref":"AKSPrimalityTest.html#module-dependencies"},{"type":"module","title":"References - AKSPrimalityTest","doc":"- Agrawal, M., Kayal, N., & Saxena, N. (2004). PRIMES is in P.\n  Annals of Mathematics, 781-793.","ref":"AKSPrimalityTest.html#module-references"},{"type":"function","title":"AKSPrimalityTest.of/1","doc":"Determines whether a given positive integer is prime using the AKS primality test.","ref":"AKSPrimalityTest.html#of/1"},{"type":"function","title":"Parameters - AKSPrimalityTest.of/1","doc":"- `n` - A positive integer to test for primality","ref":"AKSPrimalityTest.html#of/1-parameters"},{"type":"function","title":"Returns - AKSPrimalityTest.of/1","doc":"- `true` if n is prime\n- `false` if n is composite","ref":"AKSPrimalityTest.html#of/1-returns"},{"type":"function","title":"Examples - AKSPrimalityTest.of/1","doc":"iex> AKSPrimalityTest.of(2)\n    true\n\n    iex> AKSPrimalityTest.of(3)\n    true\n\n    iex> AKSPrimalityTest.of(4)\n    false\n\n    iex> AKSPrimalityTest.of(17)\n    true\n\n    iex> AKSPrimalityTest.of(25)\n    false","ref":"AKSPrimalityTest.html#of/1-examples"},{"type":"function","title":"Algorithm Steps - AKSPrimalityTest.of/1","doc":"1. **Special case**: 2 is always prime\n2. **Perfect power test**: If n is a perfect power, it's composite\n3. **Order finding**: Find suitable parameter r for polynomial test\n4. **GCD verification**: Ensure n is coprime with numbers up to r\n5. **Polynomial congruence**: Final verification using polynomial arithmetic","ref":"AKSPrimalityTest.html#of/1-algorithm-steps"},{"type":"extras","title":"AKS Primality Test","doc":"# AKS Primality Test\n\n[![Hex.pm](https://img.shields.io/hexpm/v/a_k_s_primality_test)](https://hex.pm/packages/a_k_s_primality_test)\n[![Hex.pm](https://img.shields.io/hexpm/dt/a_k_s_primality_test)](https://hex.pm/packages/a_k_s_primality_test)\n[![Hex.pm](https://img.shields.io/hexpm/l/a_k_s_primality_test)](https://hex.pm/packages/a_k_s_primality_test)\n\nA pure Elixir implementation of the AKS (Agrawal-Kayal-Saxena) primality test algorithm.","ref":"readme.html"},{"type":"extras","title":"Overview - AKS Primality Test","doc":"The AKS primality test is a deterministic polynomial-time algorithm for determining whether a given number is prime. It was the first polynomial-time primality test that is both general and deterministic, proving that the primality problem is in P.","ref":"readme.html#overview"},{"type":"extras","title":"Features - AKS Primality Test","doc":"- **Deterministic**: Always gives the correct answer (no probabilistic uncertainty)\n- **Polynomial Time**: Runs in $O((\\log n)^{12})$ time\n- **Pure Elixir**: No external dependencies for the core algorithm\n- **Well Documented**: Comprehensive documentation with mathematical explanations\n- **Type Safe**: Full type specifications for all functions","ref":"readme.html#features"},{"type":"extras","title":"Installation - AKS Primality Test","doc":"The package can be installed by adding `a_k_s_primality_test` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:a_k_s_primality_test, \"~> 1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - AKS Primality Test","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Basic Usage - AKS Primality Test","doc":"```elixir\niex> AKSPrimalityTest.of(2)\ntrue\n\niex> AKSPrimalityTest.of(3)\ntrue\n\niex> AKSPrimalityTest.of(4)\nfalse\n\niex> AKSPrimalityTest.of(17)\ntrue\n\niex> AKSPrimalityTest.of(25)\nfalse\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Testing Larger Numbers - AKS Primality Test","doc":"```elixir\n# Test some known primes\niex> AKSPrimalityTest.of(97)\ntrue\n\niex> AKSPrimalityTest.of(101)\ntrue\n\n# Test some composite numbers\niex> AKSPrimalityTest.of(100)\nfalse\n\niex> AKSPrimalityTest.of(121)\nfalse\n\niex> AKSPrimalityTest.of(127)\ntrue\n\niex> AKSPrimalityTest.of(128)\nfalse\n\niex> AKSPrimalityTest.of(8191)\ntrue\n```","ref":"readme.html#testing-larger-numbers"},{"type":"extras","title":"Algorithm Overview - AKS Primality Test","doc":"The AKS algorithm works in the following steps:\n\n1. **Perfect Power Test**: Check if $n$ is a perfect power ($n = a^b$ for some $a > 1$, $b > 1$)\n2. **Order Finding**: Find the smallest $r$ such that the order of $n$ modulo $r$ is greater than $\\log^2(n)$\n3. **GCD Test**: Check that $n$ is coprime with all numbers up to $r$\n4. **Polynomial Test**: Verify that $(X + a)^n \\equiv X^n + a \\pmod{X^r - 1, n}$ for certain values of $a$","ref":"readme.html#algorithm-overview"},{"type":"extras","title":"Mathematical Foundation - AKS Primality Test","doc":"The AKS algorithm is based on the following theorem:\n\n> Let $n \\geq 2$ be an integer. If $n$ is prime, then for all $a$ coprime to $n$,\n> $(X + a)^n \\equiv X^n + a \\pmod{n}$.\n\nThe algorithm finds a suitable parameter $r$ and tests this congruence modulo $X^r - 1$.","ref":"readme.html#mathematical-foundation"},{"type":"extras","title":"Time Complexity - AKS Primality Test","doc":"The algorithm runs in $O((\\log n)^{12})$ time, making it polynomial but not practical for very large numbers compared to probabilistic tests like Miller-Rabin. However, it is theoretically important as it proves that primality testing is in P.","ref":"readme.html#time-complexity"},{"type":"extras","title":"Dependencies - AKS Primality Test","doc":"This implementation depends on the following modules:\n- `PerfectPower` - for perfect power detection\n- `LehmerGcd` - for GCD calculations  \n- `PrimeFactorization` - for prime factorization","ref":"readme.html#dependencies"},{"type":"extras","title":"Documentation - AKS Primality Test","doc":"- [API Documentation](https://hexdocs.pm/a_k_s_primality_test) - Full API reference\n- [Mathematical Background](https://hexdocs.pm/a_k_s_primality_test/AKSPrimalityTest.html) - Detailed algorithm explanation","ref":"readme.html#documentation"},{"type":"extras","title":"Performance Considerations - AKS Primality Test","doc":"While the AKS algorithm is theoretically polynomial-time, it is not designed for practical use with very large numbers. For production applications requiring primality testing, consider:\n\n- **Miller-Rabin** - Probabilistic but very fast\n- **AKS** - Deterministic but slower (this implementation)\n- **Specialized libraries** - For very large numbers","ref":"readme.html#performance-considerations"},{"type":"extras","title":"License - AKS Primality Test","doc":"Copyright (c) 2025 University of Kitakyushu\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at [http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#license"},{"type":"extras","title":"References - AKS Primality Test","doc":"- Agrawal, M., Kayal, N., & Saxena, N. (2004). PRIMES is in P. *Annals of Mathematics*, 781-793.\n- [AKS Primality Test on Wikipedia](https://en.wikipedia.org/wiki/AKS_primality_test)","ref":"readme.html#references"},{"type":"extras","title":"Changelog - AKS Primality Test","doc":"See [CHANGELOG.md](CHANGELOG.md) for a detailed history of changes.","ref":"readme.html#changelog"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n## [v1.1.0] 2025-07-21\n\n- **Performance Optimization**: Refactored the `product/4` function in `AKSPrimalityTest` module to use tuples instead of maps for better performance\n  - Replaced map-based coefficient storage with tuple-based storage\n  - Simplified the result processing by using `put_elem/3` and `elem/2` operations\n  - Removed unnecessary map-to-list conversion and sorting operations\n  - Updated version from 1.0.0 to 1.1.0 in mix.exs\n\n## [v1.0.0] 2025-07-21\n\n- **Initial Release**: Complete implementation of the AKS (Agrawal-Kayal-Saxena) primality test algorithm\n  - **Core Algorithm**: Pure Elixir implementation of the deterministic polynomial-time primality test\n  - **Perfect Power Detection**: Efficient detection of perfect powers using the `PerfectPower` module\n  - **GCD Calculations**: Fast GCD computations using the `LehmerGcd` module  \n  - **Prime Factorization**: Support for prime factorization using the `PrimeFactorization` module\n  - **Type Safety**: Full type specifications for all functions with `@spec` annotations\n  - **Comprehensive Documentation**: Detailed mathematical documentation with KaTeX math rendering\n  - **Mathematical Foundation**: Implementation based on the original AKS paper with proper polynomial arithmetic\n  - **Deterministic Results**: Always provides correct answers (no probabilistic uncertainty)\n  - **Polynomial Time Complexity**: Runs in O((log n)¹²) time\n  - **Well-Tested**: Comprehensive test suite covering various edge cases and known primes/composites","ref":"changelog.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}